//
//  GeoFenceManager.h
//
//  Copyright (c) 2013 ZENRIN DataCom CO., LTD. All Rights Reserved.
//

#import <Foundation/Foundation.h>

/* Library Version */

#define GEOFENCE_MANAGER_LIBRARY_VERSION        @"4.0.0"

/*!
 @header GeoFenceManager.h
 
 @brief This is the header file where my super-code is contained.
 
 This file contains the most important method and properties decalaration.
 
 @author ZDC
 @copyright  2017 ZENRIN DataCom CO., LTD. All Rights Reserved.
 @version    4.0.0
 */

/*!
 @brief Enum authentication environment
 */
typedef NSUInteger AuthEnv;
enum {
    /// production environment
    AuthEnv_Pro = 0,
    /// staging environment
    AuthEnv_Stg,
    /// test environment
    AuthEnv_Test,
};

/*!
 @brief Enum NSError code in checkLocation callback
 */
typedef NSUInteger ErrCode;
enum {
    /// トークン発行認証失敗
    ErrCode_GetTokenAuthFailed = 1,
    /// 認証不許可
    ErrCode_AuthNotAuthorized,
};

/*!
 @brief Geofence Library Class
 */
@interface GeoFenceManager : NSObject

/*!
 @brief Get instance of interface class
 
 @return Object of lib geofence - GeoFenceManager
 */
+ (GeoFenceManager *)defaultManager;

/*!
 @brief Check Current Location
 @description Follow Lat and Lon to get Notification và Area info suitable
 
 @param apiKey               String - ApiKey
 @param clientId           String - ClientID
 @param secret               String - Secret
 @param timeStamp          NSDate - Time stamp of Positioning information
 @param latitude           double - latitude
 @param longitude          double - longtitude
 @param horizontalAccuracy NSInteger - horizontalAccuracy
 @param error            NSString - error
 
 @return NSArray of Dictionary Notification Condition and Area suitable
 */

- (NSArray *)checkLocation:(NSString *)apiKey
                  clientId:(NSString *)clientId
                    secret:(NSString *)secret
                 timeStamp:(NSDate *)timeStamp
                  latitude:(double)latitude
                 longitude:(double)longitude
        horizontalAccuracy:(NSInteger)horizontalAccuracy
                     error:(NSError **)error NS_SWIFT_NOTHROW;

/*!
 @brief Check Current Location
 @description Follow Lat and Lon to get Notification và Area info suitable
 
 @param apiKey               String - ApiKey
 @param clientId           String - ClientID
 @param secret               String - Secret
 @param authEnv             NSInteger - authEnv
 @param timeStamp          NSDate - Time stamp of Positioning information
 @param latitude           double - latitude
 @param longitude          double - longtitude
 @param horizontalAccuracy NSInteger - horizontalAccuracy
 @param error            NSString - error
 
 @return NSArray of Dictionary Notification Condition and Area suitable
 */

- (NSArray *)checkLocation:(NSString *)apiKey
                  clientId:(NSString *)clientId
                    secret:(NSString *)secret
                   authEnv:(NSInteger)authEnv
                 timeStamp:(NSDate *)timeStamp
                  latitude:(double)latitude
                 longitude:(double)longitude
        horizontalAccuracy:(NSInteger)horizontalAccuracy
                     error:(NSError **)error NS_SWIFT_NOTHROW;

/*!
 @brief add NotificationConditionInformation
 
 @description add notification condition information from another information list
 
 @param informationList is NSArray
 
 @return NSArray Notification Condition Information Added
 */

- (NSArray *)addNotificationConditionInformation:(NSArray *)informationList;

/*!
 @brief Delete notification condition
 
 @param informationIDList List of NotificationCondition ID to delete
 
 @return List of deleted notification condition IDs
 */
- (NSArray *)removeNotificationConditionInformation:(NSArray *)informationIDList;

/*!
 @brief Get list notificationConditionInformation
 
 @param informationIDList is List of notification condition IDs to retrieve
 
 @return List of notification conditions
 */
- (NSArray *)notificationConditionInformationList:(NSArray *)informationIDList;

/*!
 @brief Add Area Information
 
 @param informationList List of area information to be added
 
 @return a list of area information that could not be added, return an empty array if all can be added
 */

- (NSArray *)addAreaInformation:(NSArray *)informationList;


/*!
 @brief Add Area Information For GeoJson
 
 @param informationList List of area information to be added
 
 @return a list of area information that could not be added, return an empty array if all can be added
 */

- (NSArray *)addAreaInformationForGeoJson:(NSArray *)informationList;

/*!
 @brief  Delete area information
 
 @param areaIDList List of area IDs to delete
 
 @return List of deleted area IDs
 */

- (NSArray *)removeAreaInformation:(NSArray *)areaIDList;

/*!
 @brief Delete all information ( both area information and notification condition information )
 */
- (void)clearAllInformation;

/*!
 @brief  Acquire area information
 
 @param areaIDList is a List of area IDs to retrieve
 
 @return List of area information
 */
- (NSArray *)areaInformationList:(NSArray *)areaIDList;

/*!
 @brief Reset area stay status and notification history. (not reset area information and notification conditions)
 */
- (void)resetGeoFenceStatus;

/*!
 @brief Version of library
 @note If lib build debug, string version will be "version number debug” (1.3.0 debug)
 
 @return String version of library
 */
- (NSString *)libraryVersion;

@end

/*! @brief GeoFenceManager Error Domain */
extern NSString * const GFlGeoFenceManagerErrorDomain;

/*! @brief Notification condition information tag */
extern NSString * const GFlResponseTagKeyCondition;
/*! @brief Area information tag */
extern NSString * const GFlResponseTagKeyArea;

/*! @brief Response */
extern NSString * const GFlResponseKeyResponse;
/*! @brief Status */
extern NSString * const GFlResponseKeyStatus;
/*! @brief Return Code */
extern NSString * const GFlResponseKeyReturnCode;
/*! @brief Error information */
extern NSString * const GFlResponseKeyErrorInfo;
/*! @brief Error message ID */
extern NSString * const GFlResponseKeyErrorMsgId;
/*! @brief Error message */
extern NSString * const GFlResponseKeyErrorMsg;
/*! @brief Number of acquired records */
extern NSString * const GFlResponseKeyRecCount;
/*! @brief Number of hits */
extern NSString * const GFlResponseKeyHitCount;
/*! @brief Acquisition result information */
extern NSString * const GFlResponseKeyResult;
/*! @brief Terminal information */
extern NSString * const GFlResponseKeyRecord;
/*! @brief Area ID */
extern NSString * const GFlResponseKeyAid;
/*! @brief Fail Key */
extern NSString * const GFlResponseKeyFail;

/*! @brief Notification condition ID */
extern NSString * const GFlResponseKeyNid;
/*! @brief Notification timing */
extern NSString * const GFlResponseKeyNtmg;
/*! @brief Notification day */
extern NSString * const GFlResponseKeyNwk;
/*! @brief Notification Start Time */
extern NSString * const GFlResponseKeyNstm;
/*! @brief Notification end time */
extern NSString * const GFlResponseKeyNetm;
/*! @brief Notification start period */
extern NSString * const GFlResponseKeyNsvd;
/*! @brief Notice end period */
extern NSString * const GFlResponseKeyNevd;
/*! @brief Number of days not to be re-notified */
extern NSString * const GFlResponseKeyNotnd;

/*! @brief Area information type */
extern NSString * const GFlResponseKeyAtp;
/*! @brief Polygon information */
extern NSString * const GFlResponseKeyPinf;
/*! @brief GeoJson形式のエリア情報*/
extern NSString * const GFlResponseKeyGinf;
/*! @brief Central Latitude */
extern NSString * const GFlResponseKeyLat;
/*! @brief Central Longtitude */
extern NSString * const GFlResponseKeyLon;
/*! @brief Radius */
extern NSString * const GFlResponseKeyRad;
/*! @brief IN Accuracy Threshold */
extern NSString * const GFlResponseKeyInlv;
/*! @brief IN determination number of times threshold */
extern NSString * const GFlResponseKeyInfnum;
/*! @brief OUT accuracy threshold */
extern NSString * const GFlResponseKeyOutlv;
/*! @brief OUT decision number threshold */
extern NSString * const GFlResponseKeyOutfnum;
/*! @brief Effective period (start) */
extern NSString * const GFlResponseKeyVst;
/*! @brief Effective period (end) */
extern NSString * const GFlResponseKeyVet;

/*! @brief Area information Last updated date and time */
extern NSString * const GFlResponseKeyAmdt;
/*! @brief Notification condition Last updated date and time */
extern NSString * const GFlResponseKeyNmdt;
/*! @brief Result of update */
extern NSString * const GFlGeoFenceManagerCheckUpdateKeyResult;  // (NSNumber(GeoFenceManagerCheckUpdateResult))
/*! @brief Updated date and time of saved area information (NSDate) */
extern NSString * const GFlGeoFenceManagerCheckUpdateKeyLibraryAreaDate;
/*! @brief Updated date and time of saved notification condition (NSDate) */
extern NSString * const GFlGeoFenceManagerCheckUpdateKeyLibraryNotificationDate;
/*! @brief Date and time of update of area information of server (NSDate) */
extern NSString * const GFlGeoFenceManagerCheckUpdateKeyServerAreaDate;
/*! @brief Update date and time of notification condition of server (NSDate) */
extern NSString * const GFlGeoFenceManagerCheckUpdateKeyServerNotificationDate;

/*@}*/

